@@ -4,13 +4,18 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-92-s.js
- * @description Strict Mode - checking 'this' (non-strict function declaration called by strict Function.prototype.call(undefined))
+ * @path ch10/10.4/10.4.3/10.4.3-1-51-s.js
+ * @description Strict Mode - checking 'this' (FunctionDeclaration with a strict directive prologue defined within an Anonymous FunctionExpression)
  * @noStrict
  */
     
 function testcase() {
-function f() { return this===fnGlobalObject();};
-return (function () {"use strict"; return f.call(undefined);})();
+return (function () {
+    function f() {
+        "use strict";
+        return typeof this;
+    }
+    return (f()==="undefined") && (this===fnGlobalObject());
+})();
 }
 runTestCase(testcase);
\ No newline at end of file
