@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /**
- * @path ch15/15.2/15.2.4/15.2.4.5/S15.2.4.5_A13.js
+ * @path ch15/15.2/15.2.4/15.2.4.7/S15.2.4.7_A13.js
  * @description Let O be the result of calling ToObject passing the this value as the argument.
  * @negative
  */
 
-Object.prototype.hasOwnProperty.call(null, 'foo');
+Object.prototype.propertyIsEnumerable.call(null, 'foo');
 
