@@ -4,14 +4,18 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.3/15.3.5/15.3.5.4/15.3.5.4_2-85gs.js
- * @description Strict mode - checking access to strict function caller from non-strict function (non-strict function declaration called by strict Function.prototype.call(null))
- * @noStrict
+ * @path ch15/15.3/15.3.5/15.3.5.4/15.3.5.4_2-15gs.js
+ * @description Strict mode - checking access to strict function caller from strict function (New'ed object from FunctionDeclaration defined within strict mode)
+ * @onlyStrict
+ * @negative TypeError
  */
 
 
-function f() { return gNonStrict();};
-(function () {"use strict"; return f.call(null);})();
+"use strict";
+function f() {
+    return gNonStrict();
+}
+new f();
 
 
 function gNonStrict() {
