@@ -4,21 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch15/15.2/15.2.3/15.2.3.12/15.2.3.12-2-b-i-1.js
- * @description Object.isFrozen returns false if 'O' contains own writable data property
+ * @path ch15/15.4/15.4.4/15.4.4.15/15.4.4.15-8-b-i-1.js
+ * @description Array.prototype.lastIndexOf - element to be retrieved is own data property on an Array-like object
  */
 
 
 function testcase() {
 
-        var obj = {};
-        Object.defineProperty(obj, "foo", {
-            value: 20,
-            writable: true,
-            configurable: false
-        });
-        Object.preventExtensions(obj);
-        return !Object.isFrozen(obj);
+        var obj = { 0: 0, 1: 1, 2: 2, length: 3 };
 
+        return Array.prototype.lastIndexOf.call(obj, 0) === 0 &&
+            Array.prototype.lastIndexOf.call(obj, 1) === 1 &&
+            Array.prototype.lastIndexOf.call(obj, 2) === 2;
     }
 runTestCase(testcase);
