@@ -151,7 +151,7 @@ void tst_qquickwidget::showHide()
     childView->setSource(testFileUrl("rectangle.qml"));
 
     window.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
     QVERIFY(childView->quickWindow()->isVisible());
     QVERIFY(childView->quickWindow()->visibility() != QWindow::Hidden);
 
@@ -167,13 +167,13 @@ void tst_qquickwidget::reparentAfterShow()
     QQuickWidget *childView = new QQuickWidget(&window);
     childView->setSource(testFileUrl("rectangle.qml"));
     window.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
 
     QScopedPointer<QQuickWidget> toplevelView(new QQuickWidget);
     toplevelView->setParent(&window);
     toplevelView->setSource(testFileUrl("rectangle.qml"));
     toplevelView->show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
 }
 
 void tst_qquickwidget::changeGeometry()
@@ -184,7 +184,7 @@ void tst_qquickwidget::changeGeometry()
     childView->setSource(testFileUrl("rectangle.qml"));
 
     window.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
 
     childView->setGeometry(100,100,100,100);
 }
@@ -331,7 +331,7 @@ void tst_qquickwidget::readback()
     view->setSource(testFileUrl("rectangle.qml"));
 
     view->show();
-    QVERIFY(QTest::qWaitForWindowExposed(view.data(), 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(view.data()));
 
     QImage img = view->grabFramebuffer();
     QVERIFY(!img.isNull());
@@ -367,7 +367,7 @@ void tst_qquickwidget::renderingSignals()
     QCOMPARE(afterRenderingSpy.size(), 0);
 
     widget.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&widget, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
 
     QTRY_VERIFY(beforeRenderingSpy.size() > 0);
     QTRY_VERIFY(beforeSyncSpy.size() > 0);
@@ -399,9 +399,9 @@ void tst_qquickwidget::reparentToNewWindow()
     QQuickWidget *qqw = new QQuickWidget(&window1);
     qqw->setSource(testFileUrl("rectangle.qml"));
     window1.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window1, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window1));
     window2.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window2, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&window2));
 
     QSignalSpy afterRenderingSpy(qqw->quickWindow(), &QQuickWindow::afterRendering);
     qqw->setParent(&window2);
@@ -442,7 +442,7 @@ void tst_qquickwidget::keyEvents()
     KeyHandlingWidget widget;
     widget.setSource(testFileUrl("rectangle.qml"));
     widget.show();
-    QVERIFY(QTest::qWaitForWindowExposed(widget.window(), 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(widget.window()));
 
     // Note: send the event to the QWindow, not the QWidget, in order
     // to simulate the full event processing chain.
@@ -470,7 +470,7 @@ void tst_qquickwidget::shortcuts()
     KeyHandlingWidget widget;
     widget.setSource(testFileUrl("rectangle.qml"));
     widget.show();
-    QVERIFY(QTest::qWaitForWindowExposed(widget.window(), 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(widget.window()));
 
     // Send to the widget, verify that the QQuickWindow sees it.
 
@@ -487,14 +487,14 @@ void tst_qquickwidget::enterLeave()
 {
     QQuickWidget view;
     view.setSource(testFileUrl("enterleave.qml"));
-
-    // Ensure it is not inside the window first
-    QCursor::setPos(QPoint(50, 50));
-    QTRY_VERIFY(QCursor::pos() == QPoint(50, 50));
-
     view.move(100, 100);
+
+    // Ensure the cursor is away from the window first
+    QPoint outside = view.geometry().topRight() + QPoint(100, 100);
+    QCursor::setPos(outside);
+
     view.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&view, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
     QQuickItem *rootItem = view.rootObject();
     QVERIFY(rootItem);
 
@@ -503,7 +503,7 @@ void tst_qquickwidget::enterLeave()
     QCursor::setPos(view.pos() + QPoint(50, 50));
     QTRY_VERIFY(rootItem->property("hasMouse").toBool());
     // Now check the leave
-    QCursor::setPos(view.pos() - QPoint(50, 50));
+    QCursor::setPos(outside);
     QTRY_VERIFY(!rootItem->property("hasMouse").toBool());
 }
 
@@ -515,7 +515,7 @@ void tst_qquickwidget::mouseEventWindowPos()
     quick->setSource(testFileUrl("mouse.qml"));
     quick->move(50, 50);
     widget.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&widget, 5000));
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
     QQuickItem *rootItem = quick->rootObject();
     QVERIFY(rootItem);
 
